import com.tencent.subscribe.avro.Record;
import org.apache.avro.io.Decoder;
import org.apache.avro.io.DecoderFactory;
import org.apache.avro.io.Encoder;
import org.apache.avro.io.EncoderFactory;
import org.apache.avro.specific.SpecificDatumReader;
import org.apache.avro.specific.SpecificDatumWriter;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.ByteArrayOutputStream;

/**
 * @author dingweiran
 * @version 2022/6/1
 */
public class Deserializer {
    private static final Logger log = LoggerFactory.getLogger(Deserializer.class);
    private static final SpecificDatumReader<Record> reader = new SpecificDatumReader<>(Record.class);
    private static final SpecificDatumWriter<Record> writer = new SpecificDatumWriter<>(Record.class);


    public static byte[] avroSerialize(Record record) {
        try (ByteArrayOutputStream baos = new ByteArrayOutputStream()) {
            Encoder encoder = EncoderFactory.get().binaryEncoder(baos, null);
            writer.write(record, encoder);
            return baos.toByteArray();
        } catch (Throwable ex) {
            log.error("AvroSerializer: serialize record failed cause " + ex.getMessage(), ex);
            throw new RuntimeException(ex);
        }
    }

    public static Record avroDeserialize(byte[] data) {
        Decoder decoder = DecoderFactory.get().binaryDecoder(data, null);
        Record payload = null;
        try {
            payload = reader.read(null, decoder);
            return payload;
        } catch (Throwable ex) {
            log.error("AvroDeserializer: deserialize record failed cause " + ex.getMessage(), ex);
            throw new RuntimeException(ex);
        }
    }
}
