import com.tencent.subscribe.avro.Record;
import org.apache.flink.api.common.typeinfo.TypeInformation;
import org.apache.flink.streaming.connectors.kafka.KafkaDeserializationSchema;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.apache.kafka.common.header.Header;

import java.util.Iterator;

public class RecordSchema implements KafkaDeserializationSchema<RecordMsgObject> {

    private final static String partitionSeqKey = "ps";

    @Override
    public boolean isEndOfStream(RecordMsgObject obj) {
        return false;
    }

    @Override
    public RecordMsgObject deserialize(ConsumerRecord<byte[], byte[]> record) throws Exception {
        RecordMsgObject obj = new RecordMsgObject();
        obj.topic = record.topic();
        obj.partition = record.partition();
        obj.offset = record.offset();
        obj.partitionSeq = getPartitionSeq(record);
        obj.key = new String(record.key());
        obj.value = Deserializer.avroDeserialize(record.value());
        return obj;
    }

    @Override
    public TypeInformation<RecordMsgObject> getProducedType() {
        return TypeInformation.of(RecordMsgObject.class);
    }

    private static long getPartitionSeq(ConsumerRecord<byte[], byte[]> msg) {
        Iterator<Header> it = msg.headers().headers(partitionSeqKey).iterator();
        if (it.hasNext()) {
            Header h = it.next();
            return Long.parseUnsignedLong(new String(h.value()));
        }

        throw new IllegalStateException(partitionSeqKey + " does not exists");
    }
}